/* Copyright (c) 2009, Delft University of Technology

    This file is part of the MEGS package.

    The MEGS pakcage is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    The MEGS pakcage is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the MEGS package.  If not, see
    <http://www.gnu.org/licenses/>.
*/
package engine;

import java.text.DecimalFormat;

/**
 * A class that keeps record of the status of a global predicate * As this class shoud be called directly after a status update
 * this assumes the change is at the given time
 */
public class GlobalPredicate {
	
	
	/**
	 * Name of the global predicate 
	 */
	private String name;
	
	
	/**
	 * Last time the global predicate was checked 
	 */
	private double lastTime = 0;
	
	/**
	 * Last value the global predicate had, this assumes the systems starts
	 * with the global predicate being true
	 */
	private boolean lastValue = true;
	
	/**
	 * The total amount of time this global predicate was true
	 */
	private double amountTrue = 0;
	
	/**
	 * @param name The textual representation of the global predicate
	 */
	public GlobalPredicate(String name)
	{
		this.name = name;
	}
	
	/**
	 * Update the status of this global predicate (this assumes at least the 
	 * changes in value are reported, but more often is allowed)
	 * @param time The current time
	 * @param value The current value
	 */
	public void updatePredicate(double time, boolean value)
	{
		//if the last value was true and so is this one, then add the 
		//time difference, otherwise only update status
		// when going from true to false, also add the value as we assume the change to 
		// have hapepened recently
		if(lastValue)
		{
			amountTrue += time - lastTime; 
		}
		lastTime = time;
		lastValue = value;
	}
	
	public String toString()
	{
		DecimalFormat df = new DecimalFormat("0.000");
		return name + ": " + (df.format(100*amountTrue/lastTime)) + "% true";
	}
	
	
}
