/* Copyright (c) 2009, Delft University of Technology

    This file is part of the MEGS package.

    The MEGS pakcage is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    The MEGS pakcage is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the MEGS package.  If not, see
    <http://www.gnu.org/licenses/>.
*/
package engine;

import java.io.*; 
import java.util.*;
import java.text.DecimalFormat;

/**
 * @author menno
 * Reads the data from a single file, all events are assumed to be ordered
 * Takes usr3 output
 */
public class MultipleUARTFileReader implements EventReader {
	
	
	/**
	 * All event readers
	 */
	private SingleUARTFileReader[] readers;
	
	/**
	 * Holds all the next events
	 */
	
	private Event[] events;
	
	/**
	 * Constructs a reader from the files in  files
	 * @param files Files to be used, files must exist or a FileNotFoundException is thrown
	 */
	public MultipleUARTFileReader(File[] files) throws FileNotFoundException
	{
		readers = new SingleUARTFileReader[files.length];
		events = new Event[files.length];
		
		for(int i = 0; i < files.length ; i++)
		{
			readers[i] = new SingleUARTFileReader(files[i]);
			events[i] = readers[i].nextEvent();
		}

	}
	
	/* (non-Javadoc)
	 * @see EventReader#hasMoreEvents()
	 */
	public boolean hasMoreEvents() {
		
		//has more events if the list of events contains at least one event
		for(Event e : events)
		{
			if(e != null)
			{
				return true;
			}
		}
		
		//no loaded events
		return false;
	}

	/* (non-Javadoc)
	 * @see EventReader#nextEvent()
	 */
	public Event nextEvent() {
		
		//System.out.println("Next");
		//find the event with the lowest time
		Event event = null;
		int readerNum = -1; //the reader number that event came from
		for(int i = 0; i < events.length;i++)
		{
			//pick first event if none has been choosen yet (eventhough it can be null)
			if(event == null)
			{
				event = events[i];
				readerNum = i;
			}
			else
			{
				//replace if event[i] exists and is earlier 
				if(events[i] != null && events[i].getTime() < event.getTime())
				{
					event = events[i];
					readerNum = i;
				}
			}
		}
		
		//System.out.println("Next:" + readerNum +  " " + event + " " + eventString());
		
		if(event == null)
		{
			//no event found in any reader
			return null;
		}
		else
		{
			//select event and pick a new one
			Event e = events[readerNum];
			events[readerNum] = readers[readerNum].nextEvent();
			return e;
		}
	}
	
	/**
	 * Debug string of the loaded events
	 * @return
	 */
	private String eventString()
	{
		String s = "[";
		for(int i = 0; i < events.length; i++)
		{
			if(events[i] != null)
			{
				s += events[i].time +", ";
			}
			else
			{
				s+= "null, ";
			}
		}
		s+= "]";
		return s;
	}
	
	/**
	 * Returns all existing files represented by the filename
	 * @param filename String representing a file, all occurences of ^^ are replaced by 00-99
	 * @return
	 */
	public static File[] makeFileList(String filename)
	{
		DecimalFormat df = new DecimalFormat("00");
		Vector<File> files = new Vector<File>(); 
		for(int i = 0; i <=99; i++)
		{
			
			File f = new File(filename.replace("^^", df.format(i)));
			if(f.exists())
			{
				System.out.println("Found: " + f.getName());
				files.add(f);
			}
			
		}
		return files.toArray(new File[0]);
		
	}

}
