/* Copyright (c) 2009, Delft University of Technology

    This file is part of the MEGS package.

    The MEGS pakcage is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    The MEGS pakcage is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the MEGS package.  If not, see
    <http://www.gnu.org/licenses/>.
*/
package engine;

import java.text.DecimalFormat;

/**
 * Describes an assert that must be true at some point before the given time.
 *
 */
public class TimedAssert {
	//the time the assert must be true at
	private double time;
	
	//the assert to keep the record in 
	private Assert ass;
	
	private int node;
	
	/**
	 * @param time The time the assert expires
	 * @param node The node the assert operates on
	 * @param ass The assert obkject to keep the statistics in
	 */
	public TimedAssert(double time, int node, Assert ass)
	{
		this.node = node;
		this.time = time;
		this.ass = ass;
		
	}
	
	/**
	 * Returns true if the worldTime is less equal to this asserts expiration time
	 * @param worldTime
	 */
	public boolean isValid(double worldTime)
	{
		return worldTime <= time;
	}
	
	/**
	 * applies the result to the assert, should be called only once 
	 */
	public void assertResult( boolean result)
	{
		if(result)
		{
			ass.countSucces();
		}
		else
		{
			ass.countFailed();
		}
	}
	
	/**
	 * Returns the number that goes with this timed assert
	 * @return
	 */
	public int getAssertNumber()
	{
		return ass.number;
	}

	public int getNode() {
		return node;
	}
	
	/**
	 * Gives a message that represents the assert in a failed stated
	 * @return
	 */
	public String getFailMessage()
	{
		DecimalFormat df = new DecimalFormat("0.000");
		return "TimedAssert FAILED: " + node + " - " + df.format(time) + " : Assert : " + ass.number + " : " + ass.message;
	}
	
}
